@@ -56,10 +56,8 @@
 #include <nsIDOMElement.h>
 #include <nsIDOM3Node.h>
 #include <nsIIOService.h>
-#include <nsIURI.h>
 #include <nsITimer.h>
 #include <nsIComponentManager.h>
-#include <nsIServiceManager.h>
 #include <nsIProtocolHandler.h>
 #include <nsIExternalProtocolHandler.h>
 
@@ -132,6 +130,7 @@ totemPlugin::totemPlugin (NPP aInstance)
 	mWidth (-1),
 	mHeight (-1),
 	mViewerFD (-1),
+	mState (TOTEM_STATE_STOPPED),
 #ifdef TOTEM_COMPLEX_PLUGIN
 	mAutostart (PR_FALSE),
 #else
@@ -228,6 +227,26 @@ totemPlugin::DoCommand (const char *aCommand)
 	return NS_OK;
 }
 
+nsresult
+totemPlugin::SetVolume (gdouble aVolume)
+{
+	D ("SetVolume '%f'", aVolume);
+
+	/* FIXME: queue the action instead */
+	if (!mViewerReady)
+		return NS_OK;
+
+	NS_ASSERTION (mViewerProxy, "No viewer proxy");
+	dbus_g_proxy_call_no_reply (mViewerProxy,
+				    "SetVolume",
+				    G_TYPE_DOUBLE, aVolume,
+				    G_TYPE_INVALID,
+				    G_TYPE_INVALID);
+
+	return NS_OK;
+
+}
+
 /* Viewer interaction */
 
 NPError
@@ -287,6 +306,8 @@ totemPlugin::ViewerFork ()
 	g_ptr_array_add (arr, g_strdup ("narrowspace"));
 #elif defined(TOTEM_MULLY_PLUGIN)
 	g_ptr_array_add (arr, g_strdup ("mully"));
+#elif defined(TOTEM_CONE_PLUGIN)
+	g_ptr_array_add (arr, g_strdup ("cone"));
 #else
 #error Unknown plugin type
 #endif
@@ -436,6 +457,21 @@ totemPlugin::ViewerSetup ()
 				     reinterpret_cast<void*>(this),
 				     NULL);
 
+	dbus_g_object_register_marshaller
+		(totempluginviewer_marshal_VOID__UINT_UINT_STRING,
+		 G_TYPE_NONE, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID);
+	dbus_g_proxy_add_signal (mViewerProxy,
+				 "Tick",
+				 G_TYPE_UINT,
+				 G_TYPE_UINT,
+				 G_TYPE_STRING,
+				 G_TYPE_INVALID);
+	dbus_g_proxy_connect_signal (mViewerProxy,
+				     "Tick",
+				     G_CALLBACK (TickCallback),
+				     reinterpret_cast<void*>(this),
+				     NULL);
+
 	if (mHidden) {
 		ViewerReady ();
 	} else {
@@ -477,6 +513,10 @@ totemPlugin::ViewerCleanup ()
 						"StopStream",
 						G_CALLBACK (StopStreamCallback),
 						reinterpret_cast<void*>(this));
+		dbus_g_proxy_disconnect_signal (mViewerProxy,
+						"Tick",
+						G_CALLBACK (TickCallback),
+						reinterpret_cast<void*>(this));
 
 		g_object_unref (mViewerProxy);
 		mViewerProxy = NULL;
@@ -876,6 +916,32 @@ totemPlugin::StopStreamCallback (DBusGProxy *proxy,
 	plugin->UnsetStream ();
 }
 
+/* static */ void PR_CALLBACK
+totemPlugin::TickCallback (DBusGProxy *proxy,
+			   guint aTime,
+			   guint aDuration,
+			   char *aState,
+			   void *aData)
+{
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
+	TotemStates state;
+	guint i;
+
+	NS_ASSERTION (aState != NULL, "aState is NULL probably garbage");
+
+	DD ("Tick signal received, aState %s, aTime %d, aDuration %d", aState, aTime, aDuration);
+
+	for (i = 0; i < TOTEM_STATE_INVALID; i++) {
+		if (strcmp (aState, totem_states[i]) == 0) {
+			plugin->mState = (TotemStates) i;
+			break;
+		}
+	}
+
+	plugin->mTime = aTime;
+	plugin->mDuration = aDuration;
+}
+
 /* static */ void PR_CALLBACK
 totemPlugin::ViewerSetWindowCallback (DBusGProxy *aProxy,
 				      DBusGProxyCall *aCall,
@@ -1060,7 +1126,7 @@ totemPlugin::ParseBoolean (const char *key,
         if (endptr != value && errno == 0) {
                 return num > 0;
         }
-        
+
 	D ("Unknown value '%s' for parameter '%s'", value, key);
 
 	return default_val;
@@ -1122,6 +1188,12 @@ totemPlugin::SetSrc (const nsACString& aURL)
 	if (NS_FAILED (rv)) {
 		D ("Failed to create src URI (rv=%x)", rv);
 		mSrcURI = nsnull;
+	} else {
+		if (mAutostart) {
+			RequestStream (PR_FALSE);
+		} else {
+			mWaitingForButtonPress = PR_TRUE;
+		}
 	}
 
 	return rv;
@@ -1638,6 +1710,17 @@ totemPlugin::Init (NPMIMEType mimetype,
 		height = strtol (value, NULL, 0);
 	}
 
+#ifdef TOTEM_GMP_PLUGIN
+	value = (const char *) g_hash_table_lookup (args, "vidwidth");
+	if (value != NULL) {
+		width = strtol (value, NULL, 0);
+	}
+	value = (const char *) g_hash_table_lookup (args, "vidheight");
+	if (value != NULL) {
+		height = strtol (value, NULL, 0);
+	}
+#endif /* TOTEM_GMP_PLUGIN */
+
 	/* Are we hidden? */
 	/* Treat hidden without a value as TRUE */
 	mHidden = g_hash_table_lookup (args, "hidden") != NULL &&
@@ -1725,12 +1808,14 @@ totemPlugin::Init (NPMIMEType mimetype,
 	}
 
 	mCache = GetBooleanValue (args, "cache", mCache);
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
 
+#if defined (TOTEM_NARROWSPACE_PLUGIN) || defined (TOTEM_BASIC_PLUGIN)
 	mControllerHidden = !GetBooleanValue (args, "controller", PR_TRUE);
 
 	mAutoPlay = GetBooleanValue (args, "autoplay", PR_TRUE);
 
-#endif /* TOTEM_NARROWSPACE_PLUGIN */
+#endif /* TOTEM_NARROWSPACE_PLUGIN || TOTEM_BASIC_PLUGIN */
 
 #if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
 	value = (const char *) g_hash_table_lookup (args, "console");
@@ -1812,7 +1897,7 @@ totemPlugin::Init (NPMIMEType mimetype,
 	if (height <= 16 && !mControllerHidden) {
 		mAudioOnly = PR_TRUE;
 	}
-#endif /* TOTEM_NARROWSPACE_PLUGIN */
+#endif /* TOTEM_NARROWSPACE_PLUGIN || TOTEM_BASIC_PLUGIN */
 
 #ifdef TOTEM_NARROWSPACE_PLUGIN
 	/* We need to autostart if we're using an HREF
@@ -1960,6 +2045,7 @@ totemPlugin::NewStream (NPMIMEType type,
 
 	/* To track how many data we get from ::Write */
 	mBytesStreamed = 0;
+	mBytesLength = stream->end;
 
 	return NPERR_NO_ERROR;
 }
@@ -1974,6 +2060,8 @@ totemPlugin::DestroyStream (NPStream* stream,
 	D ("DestroyStream reason %d", reason);
 
 	mStream = nsnull;
+	mBytesStreamed = 0;
+	mBytesLength = 0;
 
 	int ret = close (mViewerFD);
 	if (ret < 0) {
