@@ -192,12 +192,23 @@ totem_action_save_state (Totem *totem, const char *page_id)
 	g_free (contents);
 }
 
+static void
+totem_action_wait_force_exit (gpointer user_data)
+{
+	g_usleep (10 * G_USEC_PER_SEC);
+	exit (1);
+}
+
 void
 totem_action_exit (Totem *totem)
 {
 	GdkDisplay *display = NULL;
 	char *page_id;
 
+	/* Exit forcefully if we can't do the shutdown in 10 seconds */
+	g_thread_create ((GThreadFunc) totem_action_wait_force_exit,
+			 NULL, FALSE, NULL);
+
 	if (gtk_main_level () > 0)
 		gtk_main_quit ();
 
@@ -355,7 +366,7 @@ void
 totem_action_show_properties (Totem *totem)
 {
 	if (totem_is_fullscreen (totem) == FALSE)
-		totem_sidebar_set_current_page (totem, "properties");
+		totem_sidebar_set_current_page (totem, "properties", TRUE);
 }
 
 void
@@ -3200,7 +3211,7 @@ main (int argc, char **argv)
 	/* Initialise all the plugins, and set the default page, in case
 	 * it comes from a plugin */
 	totem_object_plugins_init (totem);
-	totem_sidebar_set_current_page (totem, sidebar_pageid);
+	totem_sidebar_set_current_page (totem, sidebar_pageid, FALSE);
 	g_free (sidebar_pageid);
 
 	if (totem->session_restored != FALSE) {
